@@ -7,6 +7,7 @@
  */
 
 
+#define IN_LIBXML
 #include "libxml.h"
 #ifdef LIBXML_HTML_ENABLED
 
@@ -102,11 +103,7 @@ found_meta:
 		    if ((attr->children != NULL) &&
 		        (attr->children->type == XML_TEXT_NODE) &&
 		        (attr->children->next == NULL)) {
-#ifndef XML_USE_BUFFER_CONTENT
 			value = attr->children->content;
-#else
-			value = xmlBufferContent(attr->children->content);
-#endif
 			if ((!xmlStrcasecmp(attr->name, BAD_CAST"http-equiv"))
 			 && (!xmlStrcasecmp(value, BAD_CAST"Content-Type")))
 			    http = 1;
@@ -242,20 +239,14 @@ found_meta:
 		xmlAttrPtr attr = cur->properties;
 		int http;
 		const xmlChar *value;
-                int same_charset;
 
 		content = NULL;
 		http = 0;
-                same_charset = 0;
 		while (attr != NULL) {
 		    if ((attr->children != NULL) &&
 		        (attr->children->type == XML_TEXT_NODE) &&
 		        (attr->children->next == NULL)) {
-#ifndef XML_USE_BUFFER_CONTENT
 			value = attr->children->content;
-#else
-			value = xmlBufferContent(attr->children->content);
-#endif
 			if ((!xmlStrcasecmp(attr->name, BAD_CAST"http-equiv"))
 			 && (!xmlStrcasecmp(value, BAD_CAST"Content-Type")))
 			    http = 1;
@@ -264,17 +255,13 @@ found_meta:
                            if ((value != NULL) && 
 				(!xmlStrcasecmp(attr->name, BAD_CAST"content")))
 			      content = value;
-			   else 
-			     if ((!xmlStrcasecmp(attr->name, BAD_CAST"charset"))
-			 		&& (!xmlStrcasecmp(value, encoding)))
-			    same_charset = 1;
                         }
-		        if ((http != 0) && (content != NULL) && (same_charset != 0))
+		        if ((http != 0) && (content != NULL))
 			    break;
 		    }
 		    attr = attr->next;
 		}
-		if ((http != 0) && (content != NULL) && (same_charset != 0)) {
+		if ((http != 0) && (content != NULL)) {
 		    meta = cur;
 		    cur = cur->next;
 		    xmlUnlinkNode(meta);
@@ -455,12 +442,7 @@ htmlNodeDumpFormat(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur,
 		 (!xmlStrEqual(cur->parent->name, BAD_CAST "script")))) {
 		xmlChar *buffer;
 
-#ifndef XML_USE_BUFFER_CONTENT
 		buffer = xmlEncodeEntitiesReentrant(doc, cur->content);
-#else
-		buffer = xmlEncodeEntitiesReentrant(doc, 
-					    xmlBufferContent(cur->content));
-#endif 
 		if (buffer != NULL) {
 		    xmlBufferWriteCHAR(buf, buffer);
 		    xmlFree(buffer);
@@ -474,11 +456,7 @@ htmlNodeDumpFormat(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur,
     if (cur->type == HTML_COMMENT_NODE) {
 	if (cur->content != NULL) {
 	    xmlBufferWriteChar(buf, "<!--");
-#ifndef XML_USE_BUFFER_CONTENT
 	    xmlBufferWriteCHAR(buf, cur->content);
-#else
-	    xmlBufferWriteCHAR(buf, xmlBufferContent(cur->content));
-#endif
 	    xmlBufferWriteChar(buf, "-->");
 	}
 	return;
@@ -490,11 +468,7 @@ htmlNodeDumpFormat(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur,
 	xmlBufferWriteCHAR(buf, cur->name);
 	if (cur->content != NULL) {
 	    xmlBufferWriteChar(buf, " ");
-#ifndef XML_USE_BUFFER_CONTENT
 	    xmlBufferWriteCHAR(buf, cur->content);
-#else
-	    xmlBufferWriteCHAR(buf, xmlBufferContent(cur->content));
-#endif
 	}
 	xmlBufferWriteChar(buf, ">");
 	return;
@@ -507,11 +481,7 @@ htmlNodeDumpFormat(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur,
     }
     if (cur->type == HTML_PRESERVE_NODE) {
 	if (cur->content != NULL) {
-#ifndef XML_USE_BUFFER_CONTENT
 	    xmlBufferWriteCHAR(buf, cur->content);
-#else
-	    xmlBufferWriteCHAR(buf, xmlBufferContent(cur->content));
-#endif
 	}
 	return;
     }
@@ -559,12 +529,7 @@ htmlNodeDumpFormat(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur,
     if ((cur->type != XML_ELEMENT_NODE) && (cur->content != NULL)) {
 	xmlChar *buffer;
 
-#ifndef XML_USE_BUFFER_CONTENT
 	buffer = xmlEncodeEntitiesReentrant(doc, cur->content);
-#else
-	buffer = xmlEncodeEntitiesReentrant(doc, 
-			    xmlBufferContent(cur->content));
-#endif
 	if (buffer != NULL) {
 	    xmlBufferWriteCHAR(buf, buffer);
 	    xmlFree(buffer);
@@ -788,9 +753,6 @@ htmlDocDumpMemory(xmlDocPtr cur, xmlChar**mem, int *size) {
  *									*
  ************************************************************************/
 
-void
-htmlDocContentDumpFormatOutput(xmlOutputBufferPtr buf, xmlDocPtr cur,
-	                       const char *encoding, int format);
 /**
  * htmlDtdDumpOutput:
  * @buf:  the HTML buffer output
@@ -953,12 +915,7 @@ htmlNodeDumpFormatOutput(xmlOutputBufferPtr buf, xmlDocPtr doc,
 		 (!xmlStrEqual(cur->parent->name, BAD_CAST "script")))) {
 		xmlChar *buffer;
 
-#ifndef XML_USE_BUFFER_CONTENT
 		buffer = xmlEncodeEntitiesReentrant(doc, cur->content);
-#else
-		buffer = xmlEncodeEntitiesReentrant(doc, 
-					    xmlBufferContent(cur->content));
-#endif 
 		if (buffer != NULL) {
 		    xmlOutputBufferWriteString(buf, (const char *)buffer);
 		    xmlFree(buffer);
@@ -972,12 +929,7 @@ htmlNodeDumpFormatOutput(xmlOutputBufferPtr buf, xmlDocPtr doc,
     if (cur->type == HTML_COMMENT_NODE) {
 	if (cur->content != NULL) {
 	    xmlOutputBufferWriteString(buf, "<!--");
-#ifndef XML_USE_BUFFER_CONTENT
 	    xmlOutputBufferWriteString(buf, (const char *)cur->content);
-#else
-	    xmlOutputBufferWriteString(buf, (const char *)
-		                       xmlBufferContent(cur->content));
-#endif
 	    xmlOutputBufferWriteString(buf, "-->");
 	}
 	return;
@@ -989,12 +941,7 @@ htmlNodeDumpFormatOutput(xmlOutputBufferPtr buf, xmlDocPtr doc,
 	xmlOutputBufferWriteString(buf, (const char *)cur->name);
 	if (cur->content != NULL) {
 	    xmlOutputBufferWriteString(buf, " ");
-#ifndef XML_USE_BUFFER_CONTENT
 	    xmlOutputBufferWriteString(buf, (const char *)cur->content);
-#else
-	    xmlOutputBufferWriteString(buf, (const char *)
-		                       xmlBufferContent(cur->content));
-#endif
 	}
 	xmlOutputBufferWriteString(buf, ">");
 	return;
@@ -1007,12 +954,7 @@ htmlNodeDumpFormatOutput(xmlOutputBufferPtr buf, xmlDocPtr doc,
     }
     if (cur->type == HTML_PRESERVE_NODE) {
 	if (cur->content != NULL) {
-#ifndef XML_USE_BUFFER_CONTENT
 	    xmlOutputBufferWriteString(buf, (const char *)cur->content);
-#else
-	    xmlOutputBufferWriteString(buf, (const char *)
-		                       xmlBufferContent(cur->content));
-#endif
 	}
 	return;
     }
@@ -1067,12 +1009,7 @@ htmlNodeDumpFormatOutput(xmlOutputBufferPtr buf, xmlDocPtr doc,
 	     * invalids to charrefs
 	     */
 
-#ifndef XML_USE_BUFFER_CONTENT
             xmlOutputBufferWriteString(buf, (const char *) cur->content);
-#else
-            xmlOutputBufferWriteString(buf, 
-		           (const char *) xmlBufferContent(cur->content));
-#endif 
     }
     if (cur->children != NULL) {
         if ((format) && (info != NULL) && (!info->isinline) &&
