@@ -25,7 +25,7 @@ extern "C" {
 #endif
 
 /*
- * The hash table
+ * The hash table.
  */
 typedef struct _xmlHashTable xmlHashTable;
 typedef xmlHashTable *xmlHashTablePtr;
@@ -33,19 +33,56 @@ typedef xmlHashTable *xmlHashTablePtr;
 /*
  * function types:
  */
+/**
+ * xmlHashDeallocator:
+ * @payload:  the data in the hash
+ * @name:  the name associated
+ *
+ * Callback to free data from a hash.
+ */
 typedef void (*xmlHashDeallocator)(void *payload, xmlChar *name);
+/**
+ * xmlHashCopier:
+ * @payload:  the data in the hash
+ * @name:  the name associated
+ *
+ * Callback to copy data from a hash.
+ *
+ * Returns a copy of the data or NULL in case of error.
+ */
 typedef void *(*xmlHashCopier)(void *payload, xmlChar *name);
-typedef void *(*xmlHashScanner)(void *payload, void *data, xmlChar *name);
+/**
+ * xmlHashScanner:
+ * @payload:  the data in the hash
+ * @data:  extra scannner data
+ * @name:  the name associated
+ *
+ * Callback when scanning data in a hash with the simple scanner.
+ */
+typedef void (*xmlHashScanner)(void *payload, void *data, xmlChar *name);
+/**
+ * xmlHashScannerFull:
+ * @payload:  the data in the hash
+ * @data:  extra scannner data
+ * @name:  the name associated
+ * @name2:  the second name associated
+ * @name3:  the third name associated
+ *
+ * Callback when scanning data in a hash with the full scanner.
+ */
+typedef void (*xmlHashScannerFull)(void *payload, void *data,
+				   const xmlChar *name, const xmlChar *name2,
+				   const xmlChar *name3);
 
 /*
- * Constructor and destructor
+ * Constructor and destructor.
  */
 xmlHashTablePtr		xmlHashCreate	(int size);
 void			xmlHashFree	(xmlHashTablePtr table,
 					 xmlHashDeallocator f);
 
 /*
- * Add a new entry to the hash table
+ * Add a new entry to the hash table.
  */
 int			xmlHashAddEntry	(xmlHashTablePtr table,
 		                         const xmlChar *name,
@@ -76,7 +113,7 @@ int			xmlHashUpdateEntry3(xmlHashTablePtr table,
 					 xmlHashDeallocator f);
 
 /*
- * Remove an entry from the hash table
+ * Remove an entry from the hash table.
  */
 int     xmlHashRemoveEntry(xmlHashTablePtr table, const xmlChar *name,
                            xmlHashDeallocator f);
@@ -87,7 +124,7 @@ int     xmlHashRemoveEntry3(xmlHashTablePtr table, const xmlChar *name,
                             xmlHashDeallocator f);
 
 /*
- * Retrieve the userdata
+ * Retrieve the userdata.
  */
 void *			xmlHashLookup	(xmlHashTablePtr table,
 					 const xmlChar *name);
@@ -100,29 +137,29 @@ void *			xmlHashLookup3	(xmlHashTablePtr table,
 					 const xmlChar *name3);
 
 /*
- * Helpers
+ * Helpers.
  */
 xmlHashTablePtr		xmlHashCopy	(xmlHashTablePtr table,
 					 xmlHashCopier f);
-int			xmlHashSize	(xmlHashTablePtr);
+int			xmlHashSize	(xmlHashTablePtr table);
 void			xmlHashScan	(xmlHashTablePtr table,
 					 xmlHashScanner f,
 					 void *data);
-void			xmlHashScan1	(xmlHashTablePtr table,
-					 const xmlChar *name,
-					 xmlHashScanner f,
-					 void *data);
-void			xmlHashScan2	(xmlHashTablePtr table,
-					 const xmlChar *name,
-					 const xmlChar *name2,
-					 xmlHashScanner f,
-					 void *data);
 void			xmlHashScan3	(xmlHashTablePtr table,
 					 const xmlChar *name,
 					 const xmlChar *name2,
 					 const xmlChar *name3,
 					 xmlHashScanner f,
 					 void *data);
+void			xmlHashScanFull	(xmlHashTablePtr table,
+					 xmlHashScannerFull f,
+					 void *data);
+void			xmlHashScanFull3(xmlHashTablePtr table,
+					 const xmlChar *name,
+					 const xmlChar *name2,
+					 const xmlChar *name3,
+					 xmlHashScannerFull f,
+					 void *data);
 #ifdef __cplusplus
 }
 #endif
