@@ -27,21 +27,21 @@ extern void xmlCheckVersion(int version);
  *
  * the version string like "1.2.3"
  */
-#define LIBXML_DOTTED_VERSION "2.4.14"
+#define LIBXML_DOTTED_VERSION "2.4.18"
 
 /**
  * LIBXML_VERSION:
  *
  * the version number: 1.2.3 value is 1002003
  */
-#define LIBXML_VERSION 20414
+#define LIBXML_VERSION 20418
 
 /**
  * LIBXML_VERSION_STRING:
  *
  * the version number string, 1.2.3 value is "1002003"
  */
-#define LIBXML_VERSION_STRING "20414"
+#define LIBXML_VERSION_STRING "20418"
 
 /**
  * LIBXML_TEST_VERSION:
@@ -49,7 +49,7 @@ extern void xmlCheckVersion(int version);
  * Macro to check that the libxml version in use is compatible with
  * the version the software has been compiled against
  */
-#define LIBXML_TEST_VERSION xmlCheckVersion(20414);
+#define LIBXML_TEST_VERSION xmlCheckVersion(20418);
 
 #if 0
 /**
@@ -139,6 +139,15 @@ extern void xmlCheckVersion(int version);
 #define LIBXML_XPTR_ENABLED
 #endif
 
+/**
+ * LIBXML_C14N_ENABLED:
+ *
+ * Whether the Canonicalization support is configured in
+ */
+#if 0
+#define LIBXML_C14N_ENABLED
+#endif
+
 /**
  * LIBXML_XINCLUDE_ENABLED:
  *
