@@ -11,18 +11,27 @@
 #include <string.h>
 #include <stdarg.h>
 
-#ifdef _WIN32
+#if defined (_WIN32) && !defined(__CYGWIN__)
 #ifdef _MSC_VER
 #include <winsock2.h>
 #pragma comment(lib, "ws2_32.lib")
 #define gettimeofday(p1,p2)
+#include <time.h>
 #else /* _MSC_VER */
 #include <sys/time.h>
 #endif /* _MSC_VER */
 #else /* _WIN32 */
+#ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
+#endif
+#ifdef HAVE_TIME_H
+#include <time.h>
+#endif
 #endif /* _WIN32 */
 
+#ifdef HAVE_SYS_TIMEB_H
+#include <sys/timeb.h>
+#endif
 
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
@@ -110,12 +119,144 @@ static int loaddtd = 0;
 static int progresult = 0;
 static int timing = 0;
 static int generate = 0;
-static struct timeval begin, end;
+static int dropdtd = 0;
 #ifdef LIBXML_CATALOG_ENABLED
 static int catalogs = 0;
 static int nocatalogs = 0;
 #endif
+static const char *output = NULL;
 
+
+/*
+ * Internal timing routines to remove the necessity to have unix-specific
+ * function calls
+ */
+
+#ifndef HAVE_GETTIMEOFDAY 
+#ifdef HAVE_SYS_TIMEB_H
+#ifdef HAVE_SYS_TIME_H
+#ifdef HAVE_FTIME
+
+int
+my_gettimeofday(struct timeval *tvp, void *tzp)
+{
+	struct timeb timebuffer;
+
+	ftime(&timebuffer);
+	if (tvp) {
+		tvp->tv_sec = timebuffer.time;
+		tvp->tv_usec = timebuffer.millitm * 1000L;
+	}
+	return (0);
+}
+#define HAVE_GETTIMEOFDAY 1
+#define gettimeofday my_gettimeofday
+
+#endif /* HAVE_FTIME */
+#endif /* HAVE_SYS_TIME_H */
+#endif /* HAVE_SYS_TIMEB_H */
+#endif /* !HAVE_GETTIMEOFDAY */
+
+#if defined(HAVE_GETTIMEOFDAY)
+static struct timeval begin, end;
+
+/*
+ * startTimer: call where you want to start timing
+ */
+static void
+startTimer(void)
+{
+    gettimeofday(&begin, NULL);
+}
+
+/*
+ * endTimer: call where you want to stop timing and to print out a
+ *           message about the timing performed; format is a printf
+ *           type argument
+ */
+static void
+endTimer(const char *format, ...)
+{
+    long msec;
+    va_list ap;
+
+    gettimeofday(&end, NULL);
+    msec = end.tv_sec - begin.tv_sec;
+    msec *= 1000;
+    msec += (end.tv_usec - begin.tv_usec) / 1000;
+
+#ifndef HAVE_STDARG_H
+#error "endTimer required stdarg functions"
+#endif
+    va_start(ap, format);
+    vfprintf(stderr, format, ap);
+    va_end(ap);
+
+    fprintf(stderr, " took %ld ms\n", msec);
+}
+#elif defined(HAVE_TIME_H)
+/*
+ * No gettimeofday function, so we have to make do with calling clock.
+ * This is obviously less accurate, but there's little we can do about
+ * that.
+ */
+#ifndef CLOCKS_PER_SEC
+#define CLOCKS_PER_SEC 100
+#endif
+
+static clock_t begin, end;
+static void
+startTimer(void)
+{
+    begin = clock();
+}
+static void
+endTimer(const char *fmt, ...)
+{
+    long msec;
+    va_list ap;
+
+    end = clock();
+    msec = ((end - begin) * 1000) / CLOCKS_PER_SEC;
+
+#ifndef HAVE_STDARG_H
+#error "endTimer required stdarg functions"
+#endif
+    va_start(ap, fmt);
+    vfprintf(stderr, fmt, ap);
+    va_end(ap);
+    fprintf(stderr, " took %ld ms\n", msec);
+}
+#else
+
+/*
+ * We don't have a gettimeofday or time.h, so we just don't do timing
+ */
+static void
+startTimer(void)
+{
+    /*
+     * Do nothing
+     */
+}
+static void
+endTimer(char *format, ...)
+{
+    /*
+     * We cannot do anything because we don't have a timing function
+     */
+#ifdef HAVE_STDARG_H
+    va_start(ap, format);
+    vfprintf(stderr, format, ap);
+    va_end(ap);
+    fprintf(stderr, " was not timed\n", msec);
+#else
+    /* We don't have gettimeofday, time or stdarg.h, what crazy world is
+     * this ?!
+     */
+#endif
+}
+#endif
 /************************************************************************
  * 									*
  * 			HTML ouput					*
@@ -371,13 +512,20 @@ xmlShellReadline(char *prompt) {
     return (line_read);
 #else
     char line_read[501];
+    char *ret;
+    int len;
 
     if (prompt != NULL)
 	fprintf(stdout, "%s", prompt);
     if (!fgets(line_read, 500, stdin))
         return(NULL);
     line_read[500] = 0;
-    return(strdup(line_read));
+    len = strlen(line_read);
+    ret = (char *) malloc(len + 1);
+    if (ret != NULL) {
+	memcpy (ret, line_read, len + 1);
+    }
+    return(ret);
 #endif
 }
 
@@ -405,7 +553,7 @@ static void parseAndPrintFile(char *filename) {
     xmlDocPtr doc = NULL, tmp;
 
     if ((timing) && (!repeat))
-	gettimeofday(&begin, NULL);
+	startTimer();
     
 
     if (filename == NULL) {
@@ -530,7 +678,7 @@ static void parseAndPrintFile(char *filename) {
 	    ctxt = xmlCreateFileParserCtxt(filename);
 
 	    if (ctxt == NULL) {	      
-	      /* If xmlCreateFileParseCtxt() return NULL something
+	      /* If xmlCreateFileParserCtxt() return NULL something
 		 strange happened so we don't want to do anything.  Do
 		 we want to print an error message here?
 		 <sven@zen.org> */
@@ -591,34 +739,37 @@ static void parseAndPrintFile(char *filename) {
     }
 
     if ((timing) && (!repeat)) {
-	long msec;
-	gettimeofday(&end, NULL);
-	msec = end.tv_sec - begin.tv_sec;
-	msec *= 1000;
-	msec += (end.tv_usec - begin.tv_usec) / 1000;
-	fprintf(stderr, "Parsing took %ld ms\n", msec);
+	endTimer("Parsing");
+    }
+
+    /*
+     * Remove DOCTYPE nodes
+     */
+    if (dropdtd) {
+	xmlDtdPtr dtd;
+
+	dtd = xmlGetIntSubset(doc);
+	if (dtd != NULL) {
+	    xmlUnlinkNode((xmlNodePtr)dtd);
+	    xmlFreeDtd(dtd);
+	}
     }
 
 #ifdef LIBXML_XINCLUDE_ENABLED
     if (xinclude) {
 	if ((timing) && (!repeat)) {
-	    gettimeofday(&begin, NULL);
+	    startTimer();
 	}
 	xmlXIncludeProcess(doc);
 	if ((timing) && (!repeat)) {
-	    long msec;
-	    gettimeofday(&end, NULL);
-	    msec = end.tv_sec - begin.tv_sec;
-	    msec *= 1000;
-	    msec += (end.tv_usec - begin.tv_usec) / 1000;
-	    fprintf(stderr, "Xinclude processing took %ld ms\n", msec);
+	    endTimer("Xinclude processing");
 	}
     }
 #endif
 
 #ifdef LIBXML_DEBUG_ENABLED
     /*
-     * shell interraction
+     * shell interaction
      */
     if (shell)  
         xmlShell(doc, filename, xmlShellReadline, stdout);
@@ -664,7 +815,7 @@ static void parseAndPrintFile(char *filename) {
 	if (!debug) {
 #endif
 	    if ((timing) && (!repeat)) {
-		gettimeofday(&begin, NULL);
+		startTimer();
 	    }
 #ifdef HAVE_SYS_MMAN_H
 	    if (memory) {
@@ -691,31 +842,47 @@ static void parseAndPrintFile(char *filename) {
 		}
 	    } else
 #endif /* HAVE_SYS_MMAN_H */
-	    if (compress)
-		xmlSaveFile("-", doc);
+	    if (compress) {
+		xmlSaveFile(output ? output : "-", doc);
+	    }
 	    else if (encoding != NULL) {
 	        if ( format ) {
-	            xmlSaveFormatFileEnc("-", doc, encoding, 1);
-		}  
+		    xmlSaveFormatFileEnc(output ? output : "-", doc, encoding, 1);
+		}
 		else {
-		    xmlSaveFileEnc("-", doc, encoding);
+		    xmlSaveFileEnc(output ? output : "-", doc, encoding);
 		}
 	    }
-	    else if (format)
-		xmlSaveFormatFile("-", doc, 1);
-	    else
-		xmlDocDump(stdout, doc);
+	    else if (format) {
+		xmlSaveFormatFile(output ? output : "-", doc, 1);
+	    }
+	    else {
+		FILE *out;
+		if (output == NULL)
+		    out = stdout;
+		else {
+		    out = fopen(output,"wb");
+		}
+		xmlDocDump(out, doc);
+
+		if (output)
+		    fclose(out);
+	    }
 	    if ((timing) && (!repeat)) {
-		long msec;
-		gettimeofday(&end, NULL);
-		msec = end.tv_sec - begin.tv_sec;
-		msec *= 1000;
-		msec += (end.tv_usec - begin.tv_usec) / 1000;
-		fprintf(stderr, "Saving took %ld ms\n", msec);
+		endTimer("Saving");
 	    }
 #ifdef LIBXML_DEBUG_ENABLED
 	} else {
-	    xmlDebugDumpDocument(stdout, doc);
+	    FILE *out;
+	    if (output == NULL)
+	        out = stdout;
+	    else {
+		out = fopen(output,"wb");
+	    }
+	    xmlDebugDumpDocument(out, doc);
+
+	    if (output)
+		fclose(out);
 	}
 #endif
     }
@@ -727,16 +894,11 @@ static void parseAndPrintFile(char *filename) {
 	xmlDtdPtr dtd;
 
 	if ((timing) && (!repeat)) {
-	    gettimeofday(&begin, NULL);
+	    startTimer();
 	}
 	dtd = xmlParseDTD(NULL, (const xmlChar *)dtdvalid); 
 	if ((timing) && (!repeat)) {
-	    long msec;
-	    gettimeofday(&end, NULL);
-	    msec = end.tv_sec - begin.tv_sec;
-	    msec *= 1000;
-	    msec += (end.tv_usec - begin.tv_usec) / 1000;
-	    fprintf(stderr, "Parsing DTD took %ld ms\n", msec);
+	    endTimer("Parsing DTD");
 	}
 	if (dtd == NULL) {
 	    xmlGenericError(xmlGenericErrorContext,
@@ -745,9 +907,11 @@ static void parseAndPrintFile(char *filename) {
 	} else {
 	    xmlValidCtxt cvp;
 	    if ((timing) && (!repeat)) {
-		gettimeofday(&begin, NULL);
+		startTimer();
 	    }
-	    cvp.userData = (void *) stderr;                                                 cvp.error    = (xmlValidityErrorFunc) fprintf;                                  cvp.warning  = (xmlValidityWarningFunc) fprintf;
+	    cvp.userData = (void *) stderr;
+	    cvp.error    = (xmlValidityErrorFunc) fprintf;
+	    cvp.warning  = (xmlValidityWarningFunc) fprintf;
 	    if (!xmlValidateDtd(&cvp, doc, dtd)) {
 		xmlGenericError(xmlGenericErrorContext,
 			"Document %s does not validate against %s\n",
@@ -755,19 +919,14 @@ static void parseAndPrintFile(char *filename) {
 		progresult = 3;
 	    }
 	    if ((timing) && (!repeat)) {
-		long msec;
-		gettimeofday(&end, NULL);
-		msec = end.tv_sec - begin.tv_sec;
-		msec *= 1000;
-		msec += (end.tv_usec - begin.tv_usec) / 1000;
-		fprintf(stderr, "Validating against DTD took %ld ms\n", msec);
+		endTimer("Validating against DTD");
 	    }
 	    xmlFreeDtd(dtd);
 	}
     } else if (postvalid) {
 	xmlValidCtxt cvp;
 	if ((timing) && (!repeat)) {
-	    gettimeofday(&begin, NULL);
+	    startTimer();
 	}
 	cvp.userData = (void *) stderr;
 	cvp.error    = (xmlValidityErrorFunc) fprintf;
@@ -778,12 +937,7 @@ static void parseAndPrintFile(char *filename) {
 	    progresult = 3;
 	}
 	if ((timing) && (!repeat)) {
-	    long msec;
-	    gettimeofday(&end, NULL);
-	    msec = end.tv_sec - begin.tv_sec;
-	    msec *= 1000;
-	    msec += (end.tv_usec - begin.tv_usec) / 1000;
-	    fprintf(stderr, "Validating took %ld ms\n", msec);
+	    endTimer("Validating");
 	}
     }
 
@@ -796,16 +950,11 @@ static void parseAndPrintFile(char *filename) {
      * free it.
      */
     if ((timing) && (!repeat)) {
-	gettimeofday(&begin, NULL);
+	startTimer();
     }
     xmlFreeDoc(doc);
     if ((timing) && (!repeat)) {
-	long msec;
-	gettimeofday(&end, NULL);
-	msec = end.tv_sec - begin.tv_sec;
-	msec *= 1000;
-	msec += (end.tv_usec - begin.tv_usec) / 1000;
-	fprintf(stderr, "Freeing took %ld ms\n", msec);
+	endTimer("Freeing");
     }
 }
 
@@ -834,9 +983,12 @@ static void usage(const char *name) {
     printf("\t--postvalid : do a posteriori validation, i.e after parsing\n");
     printf("\t--dtdvalid URL : do a posteriori validation against a given DTD\n");
     printf("\t--timing : print some timings\n");
+    printf("\t--output file or -o file: save to a given file\n");
     printf("\t--repeat : repeat 100 times, for timing or profiling\n");
     printf("\t--insert : ad-hoc test for valid insertions\n");
+#ifdef HAVE_ZLIB_H
     printf("\t--compress : turn on gzip compression of output\n");
+#endif
 #ifdef LIBXML_DOCB_ENABLED
     printf("\t--sgml : use the DocBook SGML parser\n");
 #endif
@@ -862,13 +1014,17 @@ static void usage(const char *name) {
 #ifdef LIBXML_XINCLUDE_ENABLED
     printf("\t--xinclude : do XInclude processing\n");
 #endif
-    printf("\t--loaddtd : fetch external Dtd\n");
+    printf("\t--loaddtd : fetch external DTD\n");
     printf("\t--dtdattr : loaddtd + populate the tree with inherited attributes \n");
+    printf("\t--dropdtd : remove the DOCTYPE of the input docs\n");
+    printf("\nLibxml project home page: http://xmlsoft.org/\n");
+    printf("To report bugs or get some help check: http://xmlsoft.org/bugs.html\n");
 }
 int
 main(int argc, char **argv) {
     int i, count;
     int files = 0;
+    int version = 0;
 
     if (argc <= 1) {
 	usage(argv[0]);
@@ -899,12 +1055,19 @@ main(int argc, char **argv) {
 	         (!strcmp(argv[i], "--noent")))
 	    noent++;
 	else if ((!strcmp(argv[i], "-version")) ||
-	         (!strcmp(argv[i], "--version")))
+	         (!strcmp(argv[i], "--version"))) {
 	    fprintf(stderr, "xmllint: using libxml version %s\n",
 		    xmlParserVersion);
-	else if ((!strcmp(argv[i], "-noout")) ||
+	    version = 1;
+	} else if ((!strcmp(argv[i], "-noout")) ||
 	         (!strcmp(argv[i], "--noout")))
 	    noout++;
+	else if ((!strcmp(argv[i], "-o")) ||
+	         (!strcmp(argv[i], "-output")) ||
+	         (!strcmp(argv[i], "--output"))) {
+	    i++;
+	    output = argv[i++];
+	}
 	else if ((!strcmp(argv[i], "-htmlout")) ||
 	         (!strcmp(argv[i], "--htmlout")))
 	    htmlout++;
@@ -943,6 +1106,9 @@ main(int argc, char **argv) {
 	    dtdvalid = argv[i];
 	    loaddtd++;
         }
+	else if ((!strcmp(argv[i], "-dropdtd")) ||
+	         (!strcmp(argv[i], "--dropdtd")))
+	    dropdtd++;
 	else if ((!strcmp(argv[i], "-insert")) ||
 	         (!strcmp(argv[i], "--insert")))
 	    insert++;
@@ -971,11 +1137,13 @@ main(int argc, char **argv) {
 	         (!strcmp(argv[i], "--xinclude")))
 	    xinclude++;
 #endif
+#ifdef HAVE_ZLIB_H
 	else if ((!strcmp(argv[i], "-compress")) ||
 	         (!strcmp(argv[i], "--compress"))) {
 	    compress++;
 	    xmlSetCompressMode(9);
         }
+#endif
 	else if ((!strcmp(argv[i], "-nowarning")) ||
 	         (!strcmp(argv[i], "--nowarning"))) {
 	    xmlGetWarningsDefaultValue = 0;
@@ -1066,6 +1234,11 @@ main(int argc, char **argv) {
 	         (!strcmp(argv[i], "--encode"))) {
 	    i++;
 	    continue;
+        } else if ((!strcmp(argv[i], "-o")) ||
+                   (!strcmp(argv[i], "-output")) ||
+                   (!strcmp(argv[i], "--output"))) {
+            i++;
+	    continue;
         }
 	if ((!strcmp(argv[i], "-dtdvalid")) ||
 	         (!strcmp(argv[i], "--dtdvalid"))) {
@@ -1073,8 +1246,8 @@ main(int argc, char **argv) {
 	    continue;
         }
 	if ((timing) && (repeat))
-	    gettimeofday(&begin, NULL);
-	/* Remember file names.  "-" means stding.  <sven@zen.org> */
+	    startTimer();
+	/* Remember file names.  "-" means stdin.  <sven@zen.org> */
 	if ((argv[i][0] != '-') || (strcmp(argv[i], "-") == 0)) {
 	    if (repeat) {
 		for (count = 0;count < 100 * repeat;count++)
@@ -1083,12 +1256,7 @@ main(int argc, char **argv) {
 		parseAndPrintFile(argv[i]);
 	    files ++;
 	    if ((timing) && (repeat)) {
-		long msec;
-		gettimeofday(&end, NULL);
-		msec = end.tv_sec - begin.tv_sec;
-		msec *= 1000;
-		msec += (end.tv_usec - begin.tv_usec) / 1000;
-		fprintf(stderr, "100 iterations took %ld ms\n", msec);
+		endTimer("100 iterations");
 	    }
 	}
     }
@@ -1097,11 +1265,13 @@ main(int argc, char **argv) {
     if ((htmlout) && (!nowrap)) {
 	xmlGenericError(xmlGenericErrorContext, "</body></html>\n");
     }
-    if ((files == 0) && (!generate)) {
+    if ((files == 0) && (!generate) && (version == 0)) {
 	usage(argv[0]);
     }
     xmlCleanupParser();
+#ifdef WITH_MEM_DEBUG
     xmlMemoryDump();
+#endif
 
     return(progresult);
 }
